@@ -1,5 +1,6 @@
 /* Copyright (c) 2007-2009, UNINETT AS
- * Copyright (c) 2012,2016-2017, NORDUnet A/S */
+ * Copyright (c) 2012,2016-2017, NORDUnet A/S
+ * Copyright (c) 2023, SWITCH */
 /* See LICENSE for licensing information. */
 
 #define _GNU_SOURCE
@@ -33,14 +34,14 @@
 #include "hostport.h"
 
 static void setprotoopts(struct commonprotoopts *opts);
-static char **getlistenerargs();
+static char **getlistenerargs(void);
 void *dtlslistener(void *arg);
 int dtlsconnect(struct server *server, int timeout, int reconnect);
 void *dtlsclientrd(void *arg);
-int clientradputdtls(struct server *server, unsigned char *rad);
+int clientradputdtls(struct server *server, unsigned char *rad, int radlen);
 void addserverextradtls(struct clsrvconf *conf);
-void dtlssetsrcres();
-void initextradtls();
+void dtlssetsrcres(void);
+void initextradtls(void);
 
 static const struct protodefs protodefs = {
     "dtls",
@@ -77,7 +78,7 @@ static void setprotoopts(struct commonprotoopts *opts) {
     protoopts = opts;
 }
 
-static char **getlistenerargs() {
+static char **getlistenerargs(void) {
     return protoopts ? protoopts->listenargs : NULL;
 }
 
@@ -87,151 +88,13 @@ struct dtlsservernewparams {
     SSL *ssl;
 };
 
-void dtlssetsrcres() {
+void dtlssetsrcres(void) {
     if (!srcres)
 	srcres =
             resolvepassiveaddrinfo(protoopts ? protoopts->sourcearg : NULL,
                                    AF_UNSPEC, NULL, protodefs.socktype);
 }
 
-unsigned char *raddtlsget(SSL *ssl, int timeout, pthread_mutex_t *lock) {
-    int cnt, len;
-    unsigned char buf[4], *rad;
-
-    cnt = sslreadtimeout(ssl, buf, 4, timeout, lock);
-    if (cnt < 1)
-        return NULL;
-
-    len = RADLEN(buf);
-    if (len < 20) {
-        debug(DBG_ERR, "raddtlsget: length too small, malformed packet! closing connection!");
-        pthread_mutex_lock(lock);
-        SSL_shutdown(ssl);
-        pthread_mutex_unlock(lock);
-        return NULL;
-    }
-    rad = malloc(len);
-    if (!rad) {
-        debug(DBG_ERR, "raddtlsget: malloc failed");
-        return NULL;
-    }
-    memcpy(rad, buf, 4);
-
-    cnt = sslreadtimeout(ssl, rad + 4, len - 4, timeout, lock);
-    if (cnt < 1) {
-        free(rad);
-        return NULL;
-    }
-
-    debug(DBG_DBG, "raddtlsget: got %d bytes", len);
-    return rad;
-}
-
-void *dtlsserverwr(void *arg) {
-    int cnt;
-    unsigned long error;
-    struct client *client = (struct client *)arg;
-    struct gqueue *replyq;
-    struct request *reply;
-    char tmp[INET6_ADDRSTRLEN];
-
-    debug(DBG_DBG, "dtlsserverwr: starting for %s", addr2string(client->addr, tmp, sizeof(tmp)));
-    replyq = client->replyq;
-    for (;;) {
-        pthread_mutex_lock(&replyq->mutex);
-        while (!list_first(replyq->entries)) {
-            if (!SSL_get_shutdown(client->ssl)) {
-                debug(DBG_DBG, "dtlsserverwr: waiting for signal");
-                pthread_cond_wait(&replyq->cond, &replyq->mutex);
-                debug(DBG_DBG, "dtlsserverwr: got signal");
-            } else
-                break;
-        }
-
-        reply = (struct request *)list_shift(replyq->entries);
-        pthread_mutex_unlock(&replyq->mutex);
-
-        pthread_mutex_lock(&client->lock);
-        if (SSL_get_shutdown(client->ssl)) {
-            /* ssl might have changed while waiting */
-            pthread_mutex_unlock(&client->lock);
-            if (reply)
-                freerq(reply);
-            debug(DBG_DBG, "dtlsserverwr: ssl connection shutdown; exiting as requested");
-            pthread_exit(NULL);
-        }
-
-        while ((cnt = SSL_write(client->ssl, reply->replybuf, RADLEN(reply->replybuf))) <= 0) {
-            switch (SSL_get_error(client->ssl, cnt)) {
-                case SSL_ERROR_WANT_READ:
-                case SSL_ERROR_WANT_WRITE:
-                    continue;
-                default:
-                    while ((error = ERR_get_error()))
-                        debug(DBG_ERR, "dtlsserverwr: SSL: %s", ERR_error_string(error, NULL));
-                    pthread_mutex_unlock(&client->lock);
-                    freerq(reply);
-                    debug(DBG_DBG, "tlsserverwr: SSL error. exiting.");
-                    pthread_exit(NULL);
-            }
-        }
-        debug(DBG_DBG, "dtlsserverwr: sent %d bytes, Radius packet of length %d to %s",
-            cnt, RADLEN(reply->replybuf), addr2string(client->addr, tmp, sizeof(tmp)));
-        pthread_mutex_unlock(&client->lock);
-        freerq(reply);
-    }
-}
-
-void dtlsserverrd(struct client *client) {
-    struct request *rq;
-    uint8_t *buf;
-    pthread_t dtlsserverwrth;
-    char tmp[INET6_ADDRSTRLEN];
-
-    debug(DBG_DBG, "dtlsserverrd: starting for %s", addr2string(client->addr, tmp, sizeof(tmp)));
-
-    if (pthread_create(&dtlsserverwrth, &pthread_attr, dtlsserverwr, (void *)client)) {
-	debug(DBG_ERR, "dtlsserverrd: pthread_create failed");
-	return;
-    }
-
-    for (;;) {
-        buf = raddtlsget(client->ssl, IDLE_TIMEOUT * 3, &client->lock);
-        if (!buf) {
-            pthread_mutex_lock(&client->lock);
-            if (SSL_get_shutdown(client->ssl))
-                debug(DBG_ERR, "dtlsserverrd: connection from %s lost", addr2string(client->addr, tmp, sizeof(tmp)));
-            else {
-                debug(DBG_WARN, "tlsserverrd: timeout from %s, client %s (no requests), closing connection", addr2string(client->addr, tmp, sizeof(tmp)), client->conf->name);
-                SSL_shutdown(client->ssl);
-            }
-            SSL_set_shutdown(client->ssl, SSL_SENT_SHUTDOWN | SSL_RECEIVED_SHUTDOWN);
-            pthread_mutex_unlock(&client->lock);
-            break;
-        }
-        debug(DBG_DBG, "dtlsserverrd: got Radius message from %s", addr2string(client->addr, tmp, sizeof(tmp)));
-        rq = newrequest();
-        if (!rq) {
-            free(buf);
-            continue;
-        }
-        rq->buf = buf;
-        rq->from = client;
-        if (!radsrv(rq)) {
-            debug(DBG_ERR, "dtlsserverrd: message authentication/validation failed, closing connection from %s", addr2string(client->addr, tmp, sizeof(tmp)));
-            break;
-        }
-    }
-
-    /* stop writer by setting ssl to NULL and give signal in case waiting for data */
-    pthread_mutex_lock(&client->replyq->mutex);
-    pthread_cond_signal(&client->replyq->cond);
-    pthread_mutex_unlock(&client->replyq->mutex);
-    debug(DBG_DBG, "dtlsserverrd: waiting for writer to end");
-    pthread_join(dtlsserverwrth, NULL);
-    debug(DBG_DBG, "dtlsserverrd: reader for %s exiting", addr2string(client->addr, tmp, sizeof(tmp)));
-}
-
 void *dtlsservernew(void *arg) {
     struct dtlsservernewparams *params = (struct dtlsservernewparams *)arg;
     struct client *client;
@@ -296,7 +159,7 @@ void *dtlsservernew(void *arg) {
                 client->sock = s;
                 client->addr = addr_copy((struct sockaddr *)&params->addr);
                 client->ssl = params->ssl;
-                dtlsserverrd(client);
+                tlsserverrd(client);
                 removeclient(client);
             } else {
                 debug(DBG_WARN, "dtlsservernew: failed to create new client instance");
@@ -329,7 +192,7 @@ int getConnectionInfo(int socket, struct sockaddr *from, socklen_t fromlen, stru
     struct cmsghdr *ctrlhdr;
     struct msghdr msghdr;
     struct in6_pktinfo *info6;
-    struct iovec iov[] = {{NULL, 0}};
+    struct iovec iov[] = {{.iov_base = NULL, .iov_len = 0}};
 
     char tmp[48];
 
@@ -346,7 +209,7 @@ int getConnectionInfo(int socket, struct sockaddr *from, socklen_t fromlen, stru
         return ret;
     }
 
-    debug(DBG_DBG, "udp packet from %s", addr2string(from, tmp, sizeof(tmp)));
+    debug(DBG_DBG, "udp packet from: %s", addr2string(from, tmp, sizeof(tmp)));
 
     if (getsockname(socket, to, &tolen)) {
         debug(DBG_ERR, "getConnectionInfo: getsockname failed");
@@ -387,7 +250,6 @@ int getConnectionInfo(int socket, struct sockaddr *from, socklen_t fromlen, stru
 
 void *dtlslistener(void *arg) {
     int ndesc, flags, s = *(int *)arg;
-    unsigned char buf[4];
     struct sockaddr_storage from, to;
     struct dtlsservernewparams *params;
     struct pollfd fds[1];
@@ -414,16 +276,14 @@ void *dtlslistener(void *arg) {
 
         if (getConnectionInfo(s, (struct sockaddr *)&from, sizeof(from), (struct sockaddr *)&to, sizeof(to)) < 0) {
             debug(DBG_DBG, "dtlslistener: getConnectionInfo failed");
-            if (recv(s, buf, 4, 0) == -1)
-                debug(DBG_ERR, "dtlslistener: recv failed - %s", strerror(errno));
+            sock_dgram_skip(s);
             continue;
         }
 
         conf = find_clconf(handle, (struct sockaddr *)&from, NULL, NULL);
         if (!conf) {
             debug(DBG_INFO, "dtlslistener: got UDP from unknown peer %s, ignoring", addr2string((struct sockaddr *)&from, tmp, sizeof(tmp)));
-            if (recv(s, buf, 4, 0) == -1)
-                debug(DBG_ERR, "dtlslistener: recv failed - %s", strerror(errno));
+            sock_dgram_skip(s);
             continue;
         }
 
@@ -491,7 +351,7 @@ static void cleanup_connection(struct server *server) {
 
 int dtlsconnect(struct server *server, int timeout, int reconnect) {
     struct timeval socktimeout, now, start;
-    time_t wait;
+    uint32_t wait;
     int firsttry = 1;
     X509 *cert;
     SSL_CTX *ctx = NULL;
@@ -537,11 +397,11 @@ int dtlsconnect(struct server *server, int timeout, int reconnect) {
             hp = (struct hostportres *)entry->data;
             debug(DBG_INFO, "dtlsconnect: trying to open DTLS connection to server %s (%s port %s)", server->conf->name, hp->host, hp->port);
             if ((server->sock = bindtoaddr(source ? source : srcres, hp->addrinfo->ai_family, 0)) < 0) {
-                debug(DBG_ERR, "dtlsconnect: faild to bind socket for server %s (%s port %s)", server->conf->name, hp->host, hp->port);
+                debug(DBG_ERR, "dtlsconnect: failed to bind socket for server %s (%s port %s)", server->conf->name, hp->host, hp->port);
                 goto concleanup;
             }
             if (connect(server->sock, hp->addrinfo->ai_addr, hp->addrinfo->ai_addrlen)) {
-                debug(DBG_ERR, "dtlsconnect: faild to connect socket for server %s (%s port %s)", server->conf->name, hp->host, hp->port);
+                debug(DBG_ERR, "dtlsconnect: failed to connect socket for server %s (%s port %s)", server->conf->name, hp->host, hp->port);
                 goto concleanup;
             }
 
@@ -555,12 +415,28 @@ int dtlsconnect(struct server *server, int timeout, int reconnect) {
             server->ssl = SSL_new(ctx);
             pthread_mutex_unlock(&server->conf->tlsconf->lock);
             if (!server->ssl) {
-                debug(DBG_ERR, "dtlsconnect: failed to create SSL conneciton for server %s", server->conf->name);
+                debug(DBG_ERR, "dtlsconnect: failed to create SSL connection for server %s", server->conf->name);
                 goto concleanup;
             }
 
+            if (server->conf->sni) {
+                struct in6_addr tmp;
+                char *servername = server->conf->sniservername ? server->conf->sniservername : 
+                    server->conf->servername ? server->conf->servername :
+                    (inet_pton(AF_INET, hp->host, &tmp) || inet_pton(AF_INET6, hp->host, &tmp)) ? NULL : hp->host;
+                if (servername && !tlssetsni(server->ssl, servername)) {
+                    debug(DBG_ERR, "tlsconnect: set SNI %s failed", servername);
+                    goto concleanup;
+                }
+            }
+
             bio = BIO_new_dgram(server->sock, BIO_CLOSE);
             BIO_ctrl(bio, BIO_CTRL_DGRAM_SET_CONNECTED, 0, hp->addrinfo->ai_addr);
+            if (server->conf->dtlsmtu) {
+                SSL_set_options(server->ssl, SSL_OP_NO_QUERY_MTU);
+                if (!DTLS_set_link_mtu(server->ssl, server->conf->dtlsmtu))
+                    debug(DBG_WARN, "dtlsconnect: failed to set mtu %d", server->conf->dtlsmtu);
+            }
             SSL_set_bio(server->ssl, bio, bio);
             if (sslconnecttimeout(server->ssl, 5) <= 0) {
                 while ((error = ERR_get_error()))
@@ -611,54 +487,61 @@ concleanup:
     return 1;
 }
 
-int clientradputdtls(struct server *server, unsigned char *rad) {
+int clientradputdtls(struct server *server, unsigned char *rad, int radlen) {
     int cnt;
-    size_t len;
-    unsigned long error;
     struct clsrvconf *conf = server->conf;
 
+    if (radlen <= 0) {
+        debug(DBG_ERR, "clientradputdtls: invalid buffer (length)");
+        return 0;
+    }
+
     pthread_mutex_lock(&server->lock);
     if (server->state != RSP_SERVER_STATE_CONNECTED) {
         pthread_mutex_unlock(&server->lock);
         return 0;
     }
 
-    len = RADLEN(rad);
-    while ((cnt = SSL_write(server->ssl, rad, len)) <= 0) {
-        switch (SSL_get_error(server->ssl, cnt)) {
-            case SSL_ERROR_WANT_READ:
-            case SSL_ERROR_WANT_WRITE:
-                continue;
-            default:
-                while ((error = ERR_get_error()))
-                    debug(DBG_ERR, "clientradputdtls: DTLS: %s", ERR_error_string(error, NULL));
-                pthread_mutex_unlock(&server->lock);
-                return 0;
-        }
+    if ((cnt = sslwrite(server->ssl, rad, radlen, 0)) <= 0 ) {
+        pthread_mutex_unlock(&server->lock);
+        return 0;
     }
-    debug(DBG_DBG, "clientradputdtls: Sent %d bytes, Radius packet of length %zu to DTLS peer %s", cnt, len, conf->name);
+
+    debug(DBG_DBG, "clientradputdtls: Sent %d bytes, Radius packet of length %zu to DTLS peer %s", cnt, radlen, conf->name);
     pthread_mutex_unlock(&server->lock);
     return 1;
 }
 
 void *dtlsclientrd(void *arg) {
     struct server *server = (struct server *)arg;
-    unsigned char *buf;
+    unsigned char *buf = NULL;
+    struct timeval now;
+    int len = 0;
 
     for (;;) {
-    buf = raddtlsget(server->ssl, server->conf->retryinterval * (server->conf->retrycount+1), &server->lock);
-	if (!buf) {
-        if(SSL_get_shutdown(server->ssl) || (server->lostrqs && server->conf->statusserver!=RSP_STATSRV_OFF)) {
-            if (SSL_get_shutdown(server->ssl))
-                debug (DBG_WARN, "tlscleintrd: connection to server %s lost", server->conf->name);
-            else if (server->lostrqs)
-                debug (DBG_WARN, "dtlsclientrd: server %s did not respond, closing connection.", server->conf->name);
-    	    dtlsconnect(server, 0, 1);
-            server->lostrqs = 0;
+        len = radtlsget(server->ssl, server->conf->retryinterval * (server->conf->retrycount+1), &server->lock, &buf);
+        if (!buf || !len) {
+            if(SSL_get_shutdown(server->ssl) || server->lostrqs) {
+                if (SSL_get_shutdown(server->ssl))
+                    debug (DBG_WARN, "tlscleintrd: connection to server %s lost", server->conf->name);
+                else if (server->lostrqs)
+                    debug (DBG_WARN, "dtlsclientrd: server %s did not respond, closing connection.", server->conf->name);
+                if (server->dynamiclookuparg)
+                    break;
+                dtlsconnect(server, 0, 1);
+                server->lostrqs = 0;
+            }
+            if (server->dynamiclookuparg) {
+                gettimeofday(&now, NULL);
+                if (now.tv_sec - server->lastreply.tv_sec > IDLE_TIMEOUT) {
+                    debug(DBG_INFO, "tlsclientrd: idle timeout for %s", server->conf->name);
+                    break;
+                }
+            }
+            continue;
         }
-	    continue;
-	}
-	replyh(server, buf);
+        replyh(server, buf, len);
+        buf = NULL;
     }
 
     debug(DBG_INFO, "dtlsclientrd: exiting for %s", server->conf->name);
