@@ -1,5 +1,6 @@
 /* Copyright (c) 2007-2009, UNINETT AS
- * Copyright (c) 2010-2012,2016, NORDUnet A/S */
+ * Copyright (c) 2010-2012,2016, NORDUnet A/S
+ * Copyright (c) 2023, SWITCH */
 /* See LICENSE for licensing information. */
 
 #include <sys/time.h>
@@ -98,6 +99,7 @@ struct options {
     uint8_t *fticks_key;
     uint8_t ipv4only;
     uint8_t ipv6only;
+    uint8_t sni;
 };
 
 struct commonprotoopts {
@@ -110,6 +112,7 @@ struct request {
     uint32_t refcount;
 	pthread_mutex_t refmutex;
     uint8_t *buf, *replybuf;
+    int buflen, replybuflen;
     struct radmsg *msg;
     struct client *from;
     struct server *to;
@@ -139,11 +142,11 @@ struct clsrvconf {
     uint8_t type; /* RAD_UDP/RAD_TLS/RAD_TCP */
     const struct protodefs *pdef;
     char **hostsrc;
+    char *servername;
     int hostaf;
     char *portsrc;
     struct list *hostports;
     char **source;
-    char *confsecret;
     uint8_t *secret;
     int secret_len;
     char *tls;
@@ -171,6 +174,9 @@ struct clsrvconf {
     struct server *servers;
     char *fticks_viscountry;
     char *fticks_visinst;
+    uint8_t sni;
+    char *sniservername;
+    long dtlsmtu;
 };
 
 #include "tlscommon.h"
@@ -211,6 +217,7 @@ struct realm {
     char *name;
     char *message;
     uint8_t accresp;
+    uint8_t acclog;
     regex_t regex;
     uint32_t refcount;
     pthread_mutex_t refmutex;
@@ -232,30 +239,28 @@ struct protodefs {
     uint8_t retryintervalmax;
     uint8_t duplicateintervaldefault;
     void (*setprotoopts)(struct commonprotoopts *);
-    char **(*getlistenerargs)();
+    char **(*getlistenerargs)(void);
     void *(*listener)(void*);
     int (*connecter)(struct server *, int, int);
     void *(*clientconnreader)(void*);
-    int (*clientradput)(struct server *, unsigned char *);
+    int (*clientradput)(struct server *, unsigned char *, int);
     void (*addclient)(struct client *);
     void (*addserverextra)(struct clsrvconf *);
-    void (*setsrcres)();
-    void (*initextra)();
+    void (*setsrcres)(void);
+    void (*initextra)(void);
 };
 
-#define RADLEN(x) ntohs(((uint16_t *)(x))[1])
-
 struct clsrvconf *find_clconf(uint8_t type, struct sockaddr *addr, struct list_node **cur, struct hostportres **hp);
 struct clsrvconf *find_srvconf(uint8_t type, struct sockaddr *addr, struct list_node **cur);
 struct clsrvconf *find_clconf_type(uint8_t type, struct list_node **cur);
 struct client *addclient(struct clsrvconf *conf, uint8_t lock);
 void removelockedclient(struct client *client);
 void removeclient(struct client *client);
-struct gqueue *newqueue();
-struct request *newrequest();
+struct gqueue *newqueue(void);
+struct request *newrequest(void);
 void freerq(struct request *rq);
 int radsrv(struct request *rq);
-void replyh(struct server *server, unsigned char *buf);
+void replyh(struct server *server, uint8_t *buf, int buflen);
 struct addrinfo *resolve_hostport_addrinfo(uint8_t type, char *hostport);
 uint8_t *radattr2ascii(struct tlv *attr); /* TODO: mv this to radmsg? */
 extern pthread_attr_t pthread_attr;
