@@ -1,5 +1,6 @@
 /* Copyright (c) 2007-2009, UNINETT AS
- * Copyright (c) 2012, NORDUnet A/S */
+ * Copyright (c) 2012, NORDUnet A/S
+ * Copyright (c) 2023, SWITCH */
 /* See LICENSE for licensing information. */
 
 #include <signal.h>
@@ -28,12 +29,12 @@
 #include "debug.h"
 #include "util.h"
 static void setprotoopts(struct commonprotoopts *opts);
-static char **getlistenerargs();
+static char **getlistenerargs(void);
 void *tcplistener(void *arg);
 int tcpconnect(struct server *server, int timeout, int reconnect);
 void *tcpclientrd(void *arg);
-int clientradputtcp(struct server *server, unsigned char *rad);
-void tcpsetsrcres();
+int clientradputtcp(struct server *server, unsigned char *rad, int radlen);
+void tcpsetsrcres(void);
 
 static const struct protodefs protodefs = {
     "tcp",
@@ -69,11 +70,11 @@ static void setprotoopts(struct commonprotoopts *opts) {
     protoopts = opts;
 }
 
-static char **getlistenerargs() {
+static char **getlistenerargs(void) {
     return protoopts ? protoopts->listenargs : NULL;
 }
 
-void tcpsetsrcres() {
+void tcpsetsrcres(void) {
     if (!srcres)
 	srcres =
             resolvepassiveaddrinfo(protoopts ? protoopts->sourcearg : NULL,
@@ -83,7 +84,7 @@ void tcpsetsrcres() {
 int tcpconnect(struct server *server, int timeout, int reconnect) {
     struct timeval now, start;
     int firsttry = 1;
-    time_t wait;
+    uint32_t wait;
     struct addrinfo *source = NULL;
     struct list_node *entry;
     struct hostportres *hp;
@@ -177,78 +178,89 @@ int tcpreadtimeout(int s, unsigned char *buf, int num, int timeout) {
     return num;
 }
 
-/* timeout in seconds, 0 means no timeout (blocking) */
-unsigned char *radtcpget(int s, int timeout) {
+/* timeout in seconds, 0 means no timeout (blocking)
+   return 0 on timeout, <0 on error */
+int radtcpget(int s, int timeout, uint8_t **buf) {
     int cnt, len;
-    unsigned char buf[4], *rad;
+    unsigned char init_buf[4];
 
-    for (;;) {
-	cnt = tcpreadtimeout(s, buf, 4, timeout);
-	if (cnt < 1) {
-	    debug(DBG_DBG, cnt ? "radtcpget: connection lost" : "radtcpget: timeout");
-	    return NULL;
-	}
-
-	len = RADLEN(buf);
-	if (len < 4) {
-	    debug(DBG_ERR, "radtcpget: length too small");
-	    continue;
-	}
-	rad = malloc(len);
-	if (!rad) {
-	    debug(DBG_ERR, "radtcpget: malloc failed");
-	    continue;
-	}
-	memcpy(rad, buf, 4);
-
-	cnt = tcpreadtimeout(s, rad + 4, len - 4, timeout);
-	if (cnt < 1) {
-	    debug(DBG_DBG, cnt ? "radtcpget: connection lost" : "radtcpget: timeout");
-	    free(rad);
-	    return NULL;
-	}
-
-	if (len >= 20)
-	    break;
-
-	free(rad);
-	debug(DBG_WARN, "radtcpget: packet smaller than minimum radius size");
+    cnt = tcpreadtimeout(s, init_buf, 4, timeout);
+    if (cnt < 1) {
+        debug(DBG_DBG, cnt ? "radtcpget: connection lost" : "radtcpget: timeout");
+        return cnt;
     }
 
+    len = get_checked_rad_length(init_buf);
+    if (len <= 0) {
+        debug(DBG_ERR, "radtcpget: invalid message length (%d)! closing connection!", -len);
+        return len;
+    }
+
+    *buf = malloc(len);
+    if (!*buf) {
+        debug(DBG_ERR, "radtcpget: malloc failed! closing connection!");
+        return -1;
+    }
+    memcpy(*buf, init_buf, 4);
+
+    cnt = tcpreadtimeout(s, *buf + 4, len - 4, timeout);
+    if (cnt < 1) {
+        debug(DBG_DBG, cnt ? "radtcpget: connection lost" : "radtcpget: timeout");
+        free(*buf);
+        return cnt;
+    }
     debug(DBG_DBG, "radtcpget: got %d bytes", len);
-    return rad;
+    return len;
 }
 
-int clientradputtcp(struct server *server, unsigned char *rad) {
+int clientradputtcp(struct server *server, unsigned char *rad, int radlen) {
     int cnt;
-    size_t len;
     struct clsrvconf *conf = server->conf;
 
     if (server->state != RSP_SERVER_STATE_CONNECTED)
-	return 0;
-    len = RADLEN(rad);
-    if ((cnt = write(server->sock, rad, len)) <= 0) {
-	debug(DBG_ERR, "clientradputtcp: write error");
-	return 0;
+        return 0;
+    if (radlen <= 0) {
+        debug(DBG_ERR, "clientradputtcp: invalid buffer (length)");
+        return 0;
     }
-    debug(DBG_DBG, "clientradputtcp: Sent %d bytes, Radius packet of length %zu to TCP peer %s", cnt, len, conf->name);
+    if ((cnt = write(server->sock, rad, radlen)) <= 0) {
+        debug(DBG_ERR, "clientradputtcp: write error");
+        return 0;
+    }
+    debug(DBG_DBG, "clientradputtcp: Sent %d bytes, Radius packet of length %zu to TCP peer %s", cnt, radlen, conf->name);
     return 1;
 }
 
 void *tcpclientrd(void *arg) {
     struct server *server = (struct server *)arg;
     unsigned char *buf;
+    struct timeval now;
+    int len = 0;
 
     for (;;) {
-	buf = radtcpget(server->sock, server->dynamiclookuparg ? IDLE_TIMEOUT : 0);
-	if (!buf) {
-        if (server->dynamiclookuparg)
-		break;
-	    tcpconnect(server, 0, 1);
-	    continue;
-	}
+        len = radtcpget(server->sock, server->conf->retryinterval * (server->conf->retrycount+1), &buf);
+        if (!buf || len <= 0) {
+            if (len < 0 || (server->lostrqs && server->conf->statusserver!=RSP_STATSRV_OFF) ) {
+                if (len < 0)
+                    debug (DBG_WARN, "tlsclientrd: connection to server %s lost", server->conf->name);
+                else if (server->lostrqs)
+                    debug (DBG_WARN, "tlsclientrd: server %s did not respond, closing connection.", server->conf->name);
+                if (server->dynamiclookuparg)
+                    break;
+                tcpconnect(server, 0, 1);
+            }
+            if (server->dynamiclookuparg) {
+                gettimeofday(&now, NULL);
+                if (now.tv_sec - server->lastreply.tv_sec > IDLE_TIMEOUT) {
+                    debug(DBG_INFO, "tcpclientrd: idle timeout for %s", server->conf->name);
+                    break;
+                }
+            }
+            continue;
+        }
 
-	replyh(server, buf);
+        replyh(server, buf, len);
+        buf = NULL;
     }
     server->clientrdgone = 1;
     pthread_mutex_lock(&server->newrq_mutex);
@@ -283,10 +295,10 @@ void *tcpserverwr(void *arg) {
 	}
 	reply = (struct request *)list_shift(replyq->entries);
 	pthread_mutex_unlock(&replyq->mutex);
-	cnt = write(client->sock, reply->replybuf, RADLEN(reply->replybuf));
+	cnt = write(client->sock, reply->replybuf, reply->replybuflen);
 	if (cnt > 0)
 	    debug(DBG_DBG, "tcpserverwr: sent %d bytes, Radius packet of length %d to %s",
-		  cnt, RADLEN(reply->replybuf), addr2string(client->addr, tmp, sizeof(tmp)));
+		  cnt, reply->replybuflen, addr2string(client->addr, tmp, sizeof(tmp)));
 	else
 	    debug(DBG_ERR, "tcpserverwr: write error for %s", addr2string(client->addr, tmp, sizeof(tmp)));
 	freerq(reply);
@@ -295,35 +307,39 @@ void *tcpserverwr(void *arg) {
 
 void tcpserverrd(struct client *client) {
     struct request *rq;
-    uint8_t *buf;
+    uint8_t *buf = NULL;
     pthread_t tcpserverwrth;
     char tmp[INET6_ADDRSTRLEN];
+    int len = 0;
 
     debug(DBG_DBG, "tcpserverrd: starting for %s", addr2string(client->addr, tmp, sizeof(tmp)));
 
     if (pthread_create(&tcpserverwrth, &pthread_attr, tcpserverwr, (void *)client)) {
-	debug(DBG_ERR, "tcpserverrd: pthread_create failed");
-	return;
+        debug(DBG_ERR, "tcpserverrd: pthread_create failed");
+        return;
     }
 
     for (;;) {
-	buf = radtcpget(client->sock, 0);
-	if (!buf) {
-	    debug(DBG_ERR, "tcpserverrd: connection from %s lost", addr2string(client->addr, tmp, sizeof(tmp)));
-	    break;
-	}
-	debug(DBG_DBG, "tcpserverrd: got Radius message from %s", addr2string(client->addr, tmp, sizeof(tmp)));
-	rq = newrequest();
-	if (!rq) {
-	    free(buf);
-	    continue;
-	}
-	rq->buf = buf;
-	rq->from = client;
-	if (!radsrv(rq)) {
-	    debug(DBG_ERR, "tcpserverrd: message authentication/validation failed, closing connection from %s", addr2string(client->addr, tmp, sizeof(tmp)));
-	    break;
-	}
+        len = radtcpget(client->sock, 0, &buf);
+        if (!buf || !len) {
+            debug(DBG_ERR, "tcpserverrd: connection from %s lost", addr2string(client->addr, tmp, sizeof(tmp)));
+            break;
+        }
+        debug(DBG_DBG, "tcpserverrd: got Radius message from %s", addr2string(client->addr, tmp, sizeof(tmp)));
+        rq = newrequest();
+        if (!rq) {
+            free(buf);
+            buf = NULL;
+            continue;
+        }
+        rq->buf = buf;
+        rq->buflen = len;
+        rq->from = client;
+        if (!radsrv(rq)) {
+            debug(DBG_ERR, "tcpserverrd: message authentication/validation failed, closing connection from %s", addr2string(client->addr, tmp, sizeof(tmp)));
+            break;
+        }
+        buf = NULL;
     }
 
     /* stop writer by setting s to -1 and give signal in case waiting for data */
