@@ -1,5 +1,6 @@
 /* Copyright (c) 2007-2009, UNINETT AS
- * Copyright (c) 2012, NORDUnet A/S */
+ * Copyright (c) 2012, NORDUnet A/S
+ * Copyright (c) 2023, SWITCH */
 /* See LICENSE for licensing information. */
 
 #include <signal.h>
@@ -28,12 +29,12 @@
 #ifdef RADPROT_TLS
 
 static void setprotoopts(struct commonprotoopts *opts);
-static char **getlistenerargs();
+static char **getlistenerargs(void);
 void *tlslistener(void *arg);
 int tlsconnect(struct server *server, int timeout, int reconnect);
 void *tlsclientrd(void *arg);
-int clientradputtls(struct server *server, unsigned char *rad);
-void tlssetsrcres();
+int clientradputtls(struct server *server, unsigned char *rad, int radlen);
+void tlssetsrcres(void);
 
 static const struct protodefs protodefs = {
     "tls",
@@ -70,11 +71,11 @@ static void setprotoopts(struct commonprotoopts *opts) {
     protoopts = opts;
 }
 
-static char **getlistenerargs() {
+static char **getlistenerargs(void) {
     return protoopts ? protoopts->listenargs : NULL;
 }
 
-void tlssetsrcres() {
+void tlssetsrcres(void) {
     if (!srcres)
 	srcres =
             resolvepassiveaddrinfo(protoopts ? protoopts->sourcearg : NULL,
@@ -94,7 +95,7 @@ static void cleanup_connection(struct server *server) {
 
 int tlsconnect(struct server *server, int timeout, int reconnect) {
     struct timeval now, start;
-    time_t wait;
+    uint32_t wait;
     int firsttry = 1;
     X509 *cert;
     SSL_CTX *ctx = NULL;
@@ -160,9 +161,20 @@ int tlsconnect(struct server *server, int timeout, int reconnect) {
             server->ssl = SSL_new(ctx);
             pthread_mutex_unlock(&server->conf->tlsconf->lock);
             if (!server->ssl) {
-                debug(DBG_ERR, "tlsconnect: failed to create SSL conneciton for server %s", server->conf->name);
+                debug(DBG_ERR, "tlsconnect: failed to create SSL connection for server %s", server->conf->name);
                 goto concleanup;
             }
+
+            if (server->conf->sni) {
+                struct in6_addr tmp;
+                char *servername = server->conf->sniservername ? server->conf->sniservername : 
+                    server->conf->servername ? server->conf->servername :
+                    (inet_pton(AF_INET, hp->host, &tmp) || inet_pton(AF_INET6, hp->host, &tmp)) ? NULL : hp->host;
+                if (servername && !tlssetsni(server->ssl, servername)) {
+                    debug(DBG_ERR, "tlsconnect: set SNI %s failed", servername);
+                    goto concleanup;
+                }
+            }
             
             SSL_set_fd(server->ssl, server->sock);
             if (sslconnecttimeout(server->ssl, 5) <= 0) {
@@ -181,7 +193,9 @@ int tlsconnect(struct server *server, int timeout, int reconnect) {
             if (verifyconfcert(cert, server->conf, hp)) {
                 subj = getcertsubject(cert);
                 if(subj) {
-                    debug(DBG_WARN, "tlsconnect: TLS connection to %s (%s port %s), subject %s up", server->conf->name, hp->host, hp->port, subj);
+                    debug(DBG_WARN, "tlsconnect: TLS connection to %s (%s port %s), subject %s, %s with cipher %s up",
+                        server->conf->name, hp->host, hp->port, subj,
+                        SSL_get_version(server->ssl), SSL_CIPHER_get_name(SSL_get_current_cipher(server->ssl)));
                     free(subj);
                 }
                 X509_free(cert);
@@ -218,76 +232,8 @@ concleanup:
     return 1;
 }
 
-/* timeout in seconds, 0 means no timeout (blocking) */
-unsigned char *radtlsget(SSL *ssl, int timeout, pthread_mutex_t *lock) {
-    int cnt, len;
-    unsigned char buf[4], *rad;
-
-	cnt = sslreadtimeout(ssl, buf, 4, timeout, lock);
-	if (cnt < 1)
-        return NULL;
-
-	len = RADLEN(buf);
-	if (len < 20) {
-	    debug(DBG_ERR, "radtlsget: length too small, malformed packet! closing connection!");
-        pthread_mutex_lock(lock);
-        SSL_shutdown(ssl);
-        pthread_mutex_unlock(lock);
-        return NULL;
-	}
-	rad = malloc(len);
-	if (!rad) {
-	    debug(DBG_ERR, "radtlsget: malloc failed");
-	    return NULL;
-	}
-	memcpy(rad, buf, 4);
-
-	cnt = sslreadtimeout(ssl, rad + 4, len - 4, timeout, lock);
-	if (cnt < 1) {
-	    free(rad);
-	    return NULL;
-	}
-
-    debug(DBG_DBG, "radtlsget: got %d bytes", len);
-    return rad;
-}
-
-int dosslwrite(SSL *ssl, void *buf, int num, uint8_t may_block){
-    int ret;
-    unsigned long error;
-    struct pollfd fds[1];
-
-    if (!buf || !num) {
-            debug(DBG_ERR, "dosslwrite: was called with empty buffer!");
-            return -1;
-    }
-
-    if(!may_block) {
-        fds[0].fd = SSL_get_fd(ssl);
-        fds[0].events = POLLOUT;
-        if (poll(fds, 1, 0) <= 0 || fds[0].revents & (POLLERR | POLLHUP | POLLNVAL)) {
-            debug(DBG_DBG, "dosslwrite: socket not ready or buffer full; avoid blocking...");
-            return -1;
-        }
-    }
-
-    while ((ret = SSL_write(ssl, buf, num)) <= 0) {
-        switch (SSL_get_error(ssl, ret)) {
-            case SSL_ERROR_WANT_READ:
-            case SSL_ERROR_WANT_WRITE:
-                continue;
-            default:
-                while ((error = ERR_get_error()))
-                    debug(DBG_ERR, "dosslwrite: SSL: %s", ERR_error_string(error, NULL));
-                return ret;
-        }
-    }
-    return ret;
-}
-
-int clientradputtls(struct server *server, unsigned char *rad) {
+int clientradputtls(struct server *server, unsigned char *rad, int radlen) {
     int cnt;
-    size_t len;
     struct clsrvconf *conf = server->conf;
 
     pthread_mutex_lock(&server->lock);
@@ -296,46 +242,46 @@ int clientradputtls(struct server *server, unsigned char *rad) {
         return 0;
     }
 
-    len = RADLEN(rad);
-    if ((cnt = dosslwrite(server->ssl, rad, len, 0)) <= 0) {
+    if ((cnt = sslwrite(server->ssl, rad, radlen, 0)) <= 0) {
         pthread_mutex_unlock(&server->lock);
         return 0;
     }
 
-    debug(DBG_DBG, "clientradputtls: Sent %d bytes, Radius packet of length %zu to TLS peer %s", cnt, len, conf->name);
+    debug(DBG_DBG, "clientradputtls: Sent %d bytes, Radius packet of length %zu to TLS peer %s", cnt, radlen, conf->name);
     pthread_mutex_unlock(&server->lock);
     return 1;
 }
 
 void *tlsclientrd(void *arg) {
     struct server *server = (struct server *)arg;
-    unsigned char *buf;
+    unsigned char *buf = NULL;
     struct timeval now;
+    int len = 0;
 
     for (;;) {
-	buf = radtlsget(server->ssl, server->conf->retryinterval * (server->conf->retrycount+1), &server->lock);
-	if (!buf) {
-        if (SSL_get_shutdown(server->ssl) || (server->lostrqs && server->conf->statusserver!=RSP_STATSRV_OFF)) {
-            if (SSL_get_shutdown(server->ssl))
-                debug (DBG_WARN, "tlsclientrd: connection to server %s lost", server->conf->name);
-            else if (server->lostrqs)
-                debug (DBG_WARN, "tlsclientrd: server %s did not respond, closing connection.", server->conf->name);
-            if (server->dynamiclookuparg)
-                break;
-            tlsconnect(server, 0, 1);
-        }
-        if (server->dynamiclookuparg) {
-            gettimeofday(&now, NULL);
-            if (now.tv_sec - server->lastreply.tv_sec > IDLE_TIMEOUT) {
-                debug(DBG_INFO, "tlsclientrd: idle timeout for %s", server->conf->name);
-                break;
+        len = radtlsget(server->ssl, server->conf->retryinterval * (server->conf->retrycount+1), &server->lock, &buf);
+        if (!buf || !len) {
+            if (SSL_get_shutdown(server->ssl) || (server->lostrqs && server->conf->statusserver!=RSP_STATSRV_OFF)) {
+                if (SSL_get_shutdown(server->ssl))
+                    debug (DBG_WARN, "tlsclientrd: connection to server %s lost", server->conf->name);
+                else if (server->lostrqs)
+                    debug (DBG_WARN, "tlsclientrd: server %s did not respond, closing connection.", server->conf->name);
+                if (server->dynamiclookuparg)
+                    break;
+                tlsconnect(server, 0, 1);
             }
+            if (server->dynamiclookuparg) {
+                gettimeofday(&now, NULL);
+                if (now.tv_sec - server->lastreply.tv_sec > IDLE_TIMEOUT) {
+                    debug(DBG_INFO, "tlsclientrd: idle timeout for %s", server->conf->name);
+                    break;
+                }
+            }
+            continue;
         }
-        continue;
-    }
-
-	replyh(server, buf);
 
+        replyh(server, buf, len);
+        buf = NULL;
     }
     debug(DBG_INFO, "tlsclientrd: exiting for %s", server->conf->name);
     pthread_mutex_lock(&server->lock);
@@ -353,98 +299,6 @@ void *tlsclientrd(void *arg) {
     return NULL;
 }
 
-void *tlsserverwr(void *arg) {
-    int cnt;
-    struct client *client = (struct client *)arg;
-    struct gqueue *replyq;
-    struct request *reply;
-    char tmp[INET6_ADDRSTRLEN];
-
-    debug(DBG_DBG, "tlsserverwr: starting for %s", addr2string(client->addr, tmp, sizeof(tmp)));
-    replyq = client->replyq;
-    for (;;) {
-        pthread_mutex_lock(&replyq->mutex);
-        while (!list_first(replyq->entries)) {
-            if (!SSL_get_shutdown(client->ssl)) {
-                debug(DBG_DBG, "tlsserverwr: waiting for signal");
-                pthread_cond_wait(&replyq->cond, &replyq->mutex);
-                debug(DBG_DBG, "tlsserverwr: got signal");
-            } else
-                break;
-        }
-
-        reply = (struct request *)list_shift(replyq->entries);
-        pthread_mutex_unlock(&replyq->mutex);
-
-        pthread_mutex_lock(&client->lock);
-        if (SSL_get_shutdown(client->ssl)) {
-            if (reply)
-                freerq(reply);
-            pthread_mutex_unlock(&client->lock);
-            debug(DBG_DBG, "tlsserverwr: ssl connection shutdown; exiting as requested");
-            pthread_exit(NULL);
-        }
-
-        if ((cnt = dosslwrite(client->ssl, reply->replybuf, RADLEN(reply->replybuf), 0)) > 0) {
-            debug(DBG_DBG, "tlsserverwr: sent %d bytes, Radius packet of length %d to %s",
-                cnt, RADLEN(reply->replybuf), addr2string(client->addr, tmp, sizeof(tmp)));
-        }
-        pthread_mutex_unlock(&client->lock);
-    	freerq(reply);
-    }
-}
-
-void tlsserverrd(struct client *client) {
-    struct request *rq;
-    uint8_t *buf;
-    pthread_t tlsserverwrth;
-    char tmp[INET6_ADDRSTRLEN];
-
-    debug(DBG_DBG, "tlsserverrd: starting for %s", addr2string(client->addr, tmp, sizeof(tmp)));
-
-    if (pthread_create(&tlsserverwrth, &pthread_attr, tlsserverwr, (void *)client)) {
-        debug(DBG_ERR, "tlsserverrd: pthread_create failed");
-        return;
-    }
-
-    for (;;) {
-        buf = radtlsget(client->ssl, IDLE_TIMEOUT * 3, &client->lock);
-        if (!buf) {
-            pthread_mutex_lock(&client->lock);
-            if (SSL_get_shutdown(client->ssl))
-                debug(DBG_ERR, "tlsserverrd: connection from %s, client %s lost", addr2string(client->addr, tmp, sizeof(tmp)), client->conf->name);
-            else {
-                debug(DBG_WARN, "tlsserverrd: timeout from %s, client %s (no requests), closing connection", addr2string(client->addr, tmp, sizeof(tmp)), client->conf->name);
-                SSL_shutdown(client->ssl);
-            }
-            /* ensure shutdown state is set so tlsserverwr knows it can exit*/
-            SSL_set_shutdown(client->ssl, SSL_SENT_SHUTDOWN | SSL_RECEIVED_SHUTDOWN);
-            pthread_mutex_unlock(&client->lock);
-            break;
-        }
-        debug(DBG_DBG, "tlsserverrd: got Radius message from %s", addr2string(client->addr, tmp, sizeof(tmp)));
-        rq = newrequest();
-        if (!rq) {
-            free(buf);
-            continue;
-        }
-        rq->buf = buf;
-        rq->from = client;
-        if (!radsrv(rq)) {
-            debug(DBG_ERR, "tlsserverrd: message authentication/validation failed, closing connection from %s", addr2string(client->addr, tmp, sizeof(tmp)));
-            break;
-        }
-    }
-
-    /* signal writer so it can finish (based on SSL_shutdown) */
-    pthread_mutex_lock(&client->replyq->mutex);
-    pthread_cond_signal(&client->replyq->cond);
-    pthread_mutex_unlock(&client->replyq->mutex);
-    debug(DBG_DBG, "tlsserverrd: waiting for writer to end");
-    pthread_join(tlsserverwrth, NULL);
-    debug(DBG_DBG, "tlsserverrd: reader for %s exiting", addr2string(client->addr, tmp, sizeof(tmp)));
-}
-
 void *tlsservernew(void *arg) {
     int s, origflags;
     struct sockaddr_storage from;
@@ -506,8 +360,9 @@ void *tlsservernew(void *arg) {
         if (accepted_tls == conf->tlsconf && verifyconfcert(cert, conf, NULL)) {
             subj = getcertsubject(cert);
             if(subj) {
-                debug(DBG_WARN, "tlsservernew: TLS connection from %s, client %s, subject %s up",
-                    addr2string((struct sockaddr *)&from,tmp, sizeof(tmp)), conf->name, subj);
+                debug(DBG_WARN, "tlsservernew: TLS connection from %s, client %s, subject %s, %s with cipher %s up",
+                    addr2string((struct sockaddr *)&from,tmp, sizeof(tmp)), conf->name, subj,
+                    SSL_get_version(ssl), SSL_CIPHER_get_name(SSL_get_current_cipher(ssl)));
                 free(subj);
             }
             X509_free(cert);
