@@ -1,5 +1,6 @@
 /* Copyright (c) 2007-2009, UNINETT AS
- * Copyright (c) 2016, NORDUnet A/S */
+ * Copyright (c) 2016, NORDUnet A/S
+ * Copyright (c) 2023, SWITCH */
 /* See LICENSE for licensing information. */
 
 #include <openssl/ssl.h>
@@ -41,7 +42,7 @@ struct tls {
 };
 
 #if defined(RADPROT_TLS) || defined(RADPROT_DTLS)
-void sslinit();
+void sslinit(void);
 struct tls *tlsgettls(char *alt1, char *alt2);
 SSL_CTX *tlsgetctx(uint8_t type, struct tls *t);
 X509 *verifytlscert(SSL *ssl);
@@ -50,10 +51,15 @@ char *getcertsubject(X509 *cert);
 int conftls_cb(struct gconffile **cf, void *arg, char *block, char *opt, char *val);
 int addmatchcertattr(struct clsrvconf *conf, const char *match);
 void freematchcertattr(struct clsrvconf *conf);
-void tlsreloadcrls();
+void tlsreloadcrls(void);
+int tlssetsni(SSL *ssl, char *sni);
 int sslconnecttimeout(SSL *ssl, int timeout);
 int sslaccepttimeout (SSL *ssl, int timeout);
 int sslreadtimeout(SSL *ssl, unsigned char *buf, int num, int timeout, pthread_mutex_t *lock);
+int sslwrite(SSL *ssl, void *buf, int num, uint8_t blocking);
+int radtlsget(SSL *ssl, int timeout, pthread_mutex_t *lock, uint8_t **buf);
+void tlsserverrd(struct client *client);
+
 #endif
 
 /* Local Variables: */
