@@ -62,7 +62,7 @@ else
 fi
 
 if test -n "${SERVERS}" ; then
-        $PRINTCMD "server dynamic_radsec.${REALM} {\n${SERVERS}\n\ttype TLS\n}\n"
+        $PRINTCMD "server dynamic_radsec.${REALM} {\n${SERVERS}\n}\n"
         exit 0
 fi
 
