@@ -1,5 +1,6 @@
 /* Copyright (c) 2007-2009, UNINETT AS
- * Copyright (c) 2016, NORDUnet A/S */
+ * Copyright (c) 2016, NORDUnet A/S
+ * Copyright (c) 2023, SWITCH */
 /* See LICENSE for licensing information. */
 
 #include <sys/socket.h>
@@ -251,7 +252,7 @@ int connecttcp(struct addrinfo *addrinfo, struct addrinfo *src, uint16_t timeout
     return s;
 }
 
-time_t connect_wait(struct timeval attempt_start, struct timeval last_success, int firsttry) {
+uint32_t connect_wait(struct timeval attempt_start, struct timeval last_success, int firsttry) {
     struct timeval now;
 
     gettimeofday(&now, NULL);
@@ -277,6 +278,18 @@ time_t connect_wait(struct timeval attempt_start, struct timeval last_success, i
     return now.tv_sec - attempt_start.tv_sec;
 }
 
+/**
+ * @brief Skip (discard) dgram frame at front of queue
+ * 
+ * @param socket the dgram socket
+ */
+void sock_dgram_skip(int socket) {
+    uint8_t dummy;
+
+    if (recv(socket, &dummy, sizeof(dummy), MSG_DONTWAIT) == -1)
+        debug(DBG_ERR, "sock_dgram_skip: recv failed - %s", strerror(errno));
+}
+
 /* Local Variables: */
 /* c-file-style: "stroustrup" */
 /* End: */
